#include <QtCore>
#include <QDomDocument>
#include <QDomElement>

// Recursively remove tags and attributes in the inkscape/sodipodi namespace
void 
stripInkscapeTagsAndAttributes(QDomElement e)
{
	QRegExp inkScapeRE("^(inkscape|sodipodi):.*");
	QDomElement sibling = e;

	while (!sibling.isNull()) {
		//qDebug() << "node" << sibling.nodeName();
		if (sibling.hasAttributes()) {
			QDomNamedNodeMap attrs = sibling.attributes();
			QStringList al;
			for (int i = 0; i < attrs.count(); ++i) {
				QString name = attrs.item(i).nodeName();
				//qDebug() << "attr" << name << attrs.item(i).toAttr().value();
				if (inkScapeRE.exactMatch(name)) {
					al << name;
				}
			}
			for (int i = 0; i < al.count(); ++i) {
				//qDebug() << "removing attr" << .al[i];
				attrs.removeNamedItem(al[i]);
			}
			al.clear();
		}
		if (sibling.hasChildNodes()) {
			stripInkscapeTagsAndAttributes(sibling.firstChildElement());
		}
		sibling = sibling.nextSiblingElement();
	}
}

// Remove redundant styles
void 
stripElementList(QDomNodeList &l)
{
	QRegExp inkScapeRE("^(inkscape|sodipodi):.*");
	QRegExp strokeRE("stroke:\\s*none;");
	QRegExp uselessStrokeStylesRE("stroke-(width|line(cap|join)|miterlimit|dash(array|offset)|opacity):[^;]*;");
	QRegExp uselessStrokeTerminalStyleRE("stroke-(width|line(cap|join)|miterlimit|dash(array|offset)|opacity):.*$");

	QRegExp strokeDashArrayRE("stroke-dasharray:\\s*none;");
	QRegExp uselessStrokeDashArrayStylesRE("stroke-dashoffset:[^;]*;");

	QRegExp fillRE("fill:\\s*none;");
	QRegExp uselessFillStylesRE("fill-(opacity|rule):[^;]*;");
	
	QRegExp markerRE("marker:\\s*none;");
	QRegExp uselessMarkerStylesRE("marker-(start|mid|end):[^;]*;");
	for (int i = 0; i < l.count(); ++i) {
		QDomElement n = l.at(i).toElement();
		if (n.hasAttribute("style")) {
			QString style = n.attribute("style");
			if (style.contains(strokeRE)) {
				style.remove(uselessStrokeStylesRE);
				style.remove(uselessStrokeTerminalStyleRE);
			}
			if (style.contains(strokeDashArrayRE)) {
				style.remove(uselessStrokeDashArrayStylesRE);
			}
			if (style.contains(fillRE)) {
				style.remove(uselessFillStylesRE);
			}
			if (style.contains(markerRE)) {
				style.remove(uselessMarkerStylesRE);
			}
			n.setAttribute("style", style);
		}
#if 0
		QDomNamedNodeMap attrs = n.attributes();
		for (int i = 0; i < attrs.count(); ++i) {
			if (inkScapeRE.exactMatch(attrs.item(i).nodeName())) {
				attrs.removeNamedItem(attrs.item(i).nodeName());

			}
		}
#endif
	}
}

int
main(int argc, char **argv)
{
	QDomDocument d("svg");
	QFile in;
	if (!in.open(stdin, QIODevice::ReadOnly)) {
		qDebug() << "cannot open stdin";
		return -1;
	}
	if (!d.setContent(&in)) {
		in.close();
		return -1;
	}

	QDomNodeList paths = d.elementsByTagName("path");
	
	qDebug() << "found" << paths.count() << "path elements.";
	
	stripInkscapeTagsAndAttributes(d.documentElement());
	stripElementList(paths);

	QDomNodeList rects = d.elementsByTagName("rect");
	qDebug() << "found" << rects.count() << "rect elements.";

	stripElementList(rects);
	
	in.close();
	QString outContent = d.toString(0);
	
	QFile out;
	out.open(stdout, QIODevice::WriteOnly);
	QTextStream outStream(&out);

	outStream << outContent;

	out.close();
	

	
	return 0;
}
